import { createLogger } from "vite";
import colors from "picocolors";
import { execa } from "execa";
const filePathRegex = /^([\w/.-]+)\((\d+),(\d+)\)/g;
export function tscPlugin({ tscArgs } = {
    tscArgs: [],
}) {
    const controller = new AbortController();
    const cancelSignal = controller.signal;
    const logger = createLogger("info", { prefix: "[tsc]" });
    let tsProcess = null;
    const LOG_LEVELS = {
        command: "info",
        output: "info",
        ipc: "info",
        error: "error",
        duration: "info",
    };
    const execaOptions = {
        lines: true,
        gracefulCancel: true,
        cancelSignal,
        verbose(verboseLine, { message, type, result }) {
            if (["command", "ipc", "duration"].includes(type) ||
                result?.isGracefullyCanceled) {
                return;
            }
            message = message
                .replaceAll(/(\r\n|\n|\r)/gm, "")
                .replace(/^.*?\s-\s/g, "");
            if (!message) {
                return;
            }
            if (message.includes("error TS")) {
                type = "error";
                message = message.replace("error TS", colors.red("error") + " TS");
            }
            const pathMatch = filePathRegex.exec(message)?.[0];
            if (pathMatch) {
                message = message.replace(filePathRegex, (match, filePath, line, column) => colors.dim(`${filePath}(${line},${column})`));
            }
            const level = LOG_LEVELS[type] || "info";
            logger[level](message, { timestamp: true });
        },
    };
    return {
        name: "vite-plugin-tsc",
        // Hook to check if watch mode is enabled
        async configResolved(config) {
            const isWatchMode = config.command === "serve" || !!config.build.watch;
            if (!isWatchMode) {
                logger.info("building...", {
                    timestamp: true,
                });
                await execa("yarn", ["tsc", ...tscArgs], execaOptions);
                logger.info("building completed.", {
                    timestamp: true,
                });
            }
            else {
                logger.info("prebuild...", {
                    timestamp: true,
                });
                await execa("yarn", ["tsc", ...tscArgs], execaOptions);
                if (tsProcess) {
                    logger.warn("watch process already running. Skipping...", {
                        timestamp: true,
                    });
                    return;
                }
                tsProcess = execa("yarn", ["tsc", ...tscArgs, "--watch", "--preserveWatchOutput"], execaOptions);
                tsProcess.catch((error) => {
                    logger.error("kill.", {
                        timestamp: true,
                    });
                    if (!error.isGracefullyCanceled) {
                        logger.error("watch process failed.", {
                            timestamp: true,
                            error,
                        });
                    }
                });
            }
        },
        async closeBundle() {
            if (tsProcess) {
                controller.abort();
                await tsProcess.catch(() => { });
                tsProcess = null;
            }
        },
    };
}
//# sourceMappingURL=plugin.js.map